package com.callysto.devin.eliza;

import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

import android.app.Activity;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.content.pm.ResolveInfo;
import android.os.AsyncTask;
import android.os.Bundle;
import android.speech.RecognizerIntent;
import android.speech.tts.TextToSpeech;
import android.speech.tts.TextToSpeech.OnInitListener;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.TextView;

/**
 * Sample code that invokes the speech recognition intent API.
 */
public class MainActivity extends Activity implements OnClickListener, OnInitListener {

    private static final int VOICE_RECOGNITION_REQUEST_CODE = 4440;
    private static final int CHECK_FOR_TTS = 4441;
    private Button speakButton;
    private TextView query;
    private TextView response;
    private ElizaMain eliza;
    private TextToSpeech mTts;

    /**
     * Called with the activity is first created.
     */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        // Inflate our UI from its XML layout description.
        setContentView(R.layout.main);

        // Get display items for later interaction
        speakButton = (Button) findViewById(R.id.btn_speak);
        
        query = (TextView) findViewById(R.id.query);
        response = (TextView) findViewById(R.id.response);
        eliza = new ElizaMain ();
        eliza.readScript (this, "script");

        // Check to see if a recognition activity is present
        PackageManager pm = getPackageManager();
        List<ResolveInfo> activities = pm.queryIntentActivities(
                new Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH), 0);
        if (activities.size() != 0) {
            speakButton.setOnClickListener(this);
        } else {
            speakButton.setEnabled(false);
            speakButton.setText("Recognizer not present");
        }
        
        initializeTTS ();
    }//onCreate
    
    private void initializeTTS () {
        Intent checkIntent = new Intent();
        checkIntent.setAction(TextToSpeech.Engine.ACTION_CHECK_TTS_DATA);
        startActivityForResult(checkIntent, CHECK_FOR_TTS);
    }//initializeTTS

    /**
     * Handle the click on the start recognition button.
     */
    public void onClick(View v) {
        if (v.getId() == R.id.btn_speak) {
            startVoiceRecognitionActivity();
        }
    }

    /**
     * Fire an intent to start the speech recognition activity.
     */
    private void startVoiceRecognitionActivity() {
    	speakButton.setEnabled(false);
        Intent intent = new Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH);

        // Specify the calling package to identify your application
        //intent.putExtra(RecognizerIntent.EXTRA_CALLING_PACKAGE, getClass().getPackage().getName());

        // Display an hint to the user about what he should say.
        //intent.putExtra(RecognizerIntent.EXTRA_PROMPT, "Speech recognition demo");

        // Given an hint to the recognizer about what the user is going to say
        intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL,
                RecognizerIntent.LANGUAGE_MODEL_FREE_FORM);

        // Specify how many results you want to receive. The results will be sorted
        // where the first result is the one with higher confidence.
        intent.putExtra(RecognizerIntent.EXTRA_MAX_RESULTS, 1);

        startActivityForResult(intent, VOICE_RECOGNITION_REQUEST_CODE);
    }

    /**
     * Handle the results from the recognition activity.
     */
    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (requestCode == VOICE_RECOGNITION_REQUEST_CODE && resultCode == RESULT_OK) {
        	gotVoiceInput (data);
        } else if (requestCode == VOICE_RECOGNITION_REQUEST_CODE) {
            speakButton.setEnabled (true);
        } else if (requestCode == CHECK_FOR_TTS) {
            if (resultCode == TextToSpeech.Engine.CHECK_VOICE_DATA_PASS) {
                // success, create the TTS instance
                mTts = new TextToSpeech(this, this);
                mTts.setLanguage(Locale.UK);
                String greeting = this.getResources().getString(R.string.hi);
                say(greeting);
            } else {
                // missing data, install it
                Intent installIntent = new Intent();
                installIntent.setAction(
                    TextToSpeech.Engine.ACTION_INSTALL_TTS_DATA);
                startActivity(installIntent);
            }
        }//if
        super.onActivityResult(requestCode, resultCode, data);
    }//onActivityResult
    
    private void gotVoiceInput (Intent data) {
        // Fill the list view with the strings the recognizer thought it could have heard
        ArrayList<String> matches = data.getStringArrayListExtra(
                RecognizerIntent.EXTRA_RESULTS);
        String text = matches.get(0);
        query.setText(text);
        String reply = eliza.processInput (text);
        response.setText(reply);
        say (reply);
        speakButton.setEnabled (true);
    }//gotVoiceInput
    
    private void say (String s) {
    	(new Say ()).execute (s);
    }//say
    
    private class Say extends AsyncTask<String, Void, Void> {
		@Override
		protected Void doInBackground(String... str) {
			mTts.speak (str[0], TextToSpeech.QUEUE_FLUSH, null);
			return null;
		}//doInBackground
    }//ASyncTask

	@Override
	public void onInit(int arg0) {
		//ignore initialization
		return;
	}//onInit
}